@@ -84,6 +84,7 @@ static void daemonize(void) {
 	r = freopen("/dev/null", "r", stdin);
 	r = freopen("/dev/null", "w", stdout);
 	r = freopen("/dev/null", "w", stderr);
+	r = r;
 }
 
 /* Finger information */
@@ -127,17 +128,17 @@ void grab(Display* display, int grabDeviceID) {
 	unsigned char mask_data[8] = { 0,0,0,0,0,0,0,0 };
 	device_mask.mask_len = sizeof(mask_data);
 	device_mask.mask = mask_data;
-	//XISetMask(device_mask.mask, XI_TouchBegin);
-	//XISetMask(device_mask.mask, XI_TouchUpdate);
-	//XISetMask(device_mask.mask, XI_TouchEnd);
 	XISetMask(device_mask.mask, XI_Motion);
 	XISetMask(device_mask.mask, XI_ButtonPress);
 
+/* Experiments with X MT support, not working yet */
+//	XISetMask(device_mask.mask, XI_TouchBegin);
+//	XISetMask(device_mask.mask, XI_TouchUpdate);
+//	XISetMask(device_mask.mask, XI_TouchEnd);
 //	XIGrabModifiers modifiers;
 //	modifiers.modifiers = XIAnyModifier;
 //	int r = XIGrabButton(display, grabDeviceID, XIAnyButton, root, None, GrabModeSync,
 //			GrabModeAsync, True, &device_mask, 1, &modifiers);
-
 //	int r = XIGrabTouchBegin(display, grabDeviceID, root, None, &device_mask, 0, modifiers);
 
 	int r = XIGrabDevice(display, grabDeviceID, root, CurrentTime, None, GrabModeAsync, GrabModeAsync, False, &device_mask);
@@ -148,6 +149,7 @@ void grab(Display* display, int grabDeviceID) {
 
 /* Ungrab the device so input can be handled by application directly */
 void ungrab(Display* display, int grabDeviceID) {
+/* Experiments with X MT support, not working yet */
 //	XIGrabModifiers modifiers[1] = { { 0, 0 } };
 //	XIUngrabButton(display, grabDeviceID, 1, root, 1, modifiers);
 	XIUngrabDevice(display, grabDeviceID, CurrentTime);
@@ -176,9 +178,10 @@ void releaseButton() {
 	if (buttonDown) {
 		buttonDown = 0;
 		XTestFakeButtonEvent(display, 1, False, CurrentTime);
-		
-		/*printf("RELEASE!\n");
-		XDevice * dev = XOpenDevice(display, deviceID);
+		XFlush(display);
+
+/* Experiments with Pressure Sensitivity, not working yet */
+/*		XDevice * dev = XOpenDevice(display, deviceID);
 		int axes[3] = {fingerInfos[0].rawX, fingerInfos[0].rawY, fingerInfos[0].rawZ};
 		XTestFakeDeviceButtonEvent(display, dev, 1, False, axes, 3, 0);
 		XCloseDevice(display, dev);
@@ -191,6 +194,7 @@ void releaseButton() {
 void pressButton() {
 	if(!buttonDown) {
 
+/* Experiments with Pressure Sensitivity, not working yet */
 /*		XDevice * dev = XOpenDevice(display, deviceID);
 		int axes[3] = {fingerInfos[0].rawX, fingerInfos[0].rawY, fingerInfos[0].rawZ};
 		XTestFakeDeviceButtonEvent(display, dev, 1, False, axes, 3, 0);
@@ -219,13 +223,12 @@ int isButtonDown() {
 /* Moves the pointer to the given position */
 void movePointer(int x, int y, int z) {
 	/* Move pointer to center between touch points */
-	/* TODO: Find some way to send XI events to also send pressure level */
-//	printf("%i %i %i\n", x, y, z);
-//	int axes[3] = {x, y, z};
-	
-//	XDevice * dev = XOpenDevice(display, 17);
-//	XTestFakeDeviceMotionEvent(display, dev, False, 0, axes, 2, 0);
-//	XCloseDevice(display, dev);
+
+	/* Experiments with XI events, not working yet */
+	//	int axes[3] = {x, y, z};
+	//	XDevice * dev = XOpenDevice(display, 17);
+	//	XTestFakeDeviceMotionEvent(display, dev, False, 0, axes, 2, 0);
+	//	XCloseDevice(display, dev);
 
 	XTestFakeMotionEvent(display, -1, x, y, CurrentTime);
 	XFlush(display);
@@ -340,7 +343,6 @@ Window getLastChildWindow(Window w) {
 
 		if (childWindows != NULL) {
 			if(childCount > 0) {
-				//printf("%i children.\n", childCount);
 				Window child = childWindows[childCount - 1];
 				XFree(childWindows);
 				return child;
@@ -372,7 +374,6 @@ Window getActiveWindow() {
 /* Returns the active top-level window. A top-level window is one that has WM_CLASS set.
  * May also return None. */
 Window getCurrentWindow() {
-	//if(debugMode) printf("Called getCurrentWindow\n");
 
 	/* First get the window that has the input focus */
 	Window currentWindow;
@@ -676,10 +677,14 @@ void readCalibrationData(int exitOnFail) {
 				if(deviceInfo->classes[c]->type == XIValuatorClass) {
 					XIValuatorClassInfo* valuatorInfo = (XIValuatorClassInfo *) deviceInfo->classes[c];
 					if(valuatorInfo->mode == XIModeAbsolute) {
-						if(valuatorInfo->label == XInternAtom(display, "Abs X", 0)) {
+						if(valuatorInfo->label == XInternAtom(display, "Abs X", 0)
+						|| valuatorInfo->label == XInternAtom(display, "Abs MT Position X", 0)) 
+						{
 							calibMinX = valuatorInfo->min;
 							calibMaxX = valuatorInfo->max;
-						} else if(valuatorInfo->label == XInternAtom(display, "Abs Y", 0)) {
+						}
+						else if(valuatorInfo->label == XInternAtom(display, "Abs Y", 0) 							|| valuatorInfo->label == XInternAtom(display, "Abs MT Position Y", 0))
+						{
 							calibMinY = valuatorInfo->min;
 							calibMaxY = valuatorInfo->max;
 						}
@@ -770,6 +775,11 @@ void readCalibrationData(int exitOnFail) {
 
 	XFree(data2);
 
+	if(debugMode)
+	{
+		printf("Calibration: MinX: %i; MaxX: %i; MinY: %i; MaxY: %i\n", calibMinX, calibMaxX, calibMinY, calibMaxY);
+	}
+
 }
 
 
@@ -813,8 +823,6 @@ int main(int argc, char **argv) {
 		sleep(10);
 	}
 
-	/* We have two threads accessing X */
-	//XInitThreads();
 
 	/* Connect to X server */
 	if ((display = XOpenDisplay(NULL)) == NULL) {
@@ -842,9 +850,6 @@ int main(int argc, char **argv) {
 		devname = "/dev/twofingtouch";
 	}
 
-	///* Save return value of pthread_create so we only call it later if it hasn't been successfully called before. */
-	//int threadReturn = 1;
-
 	/* Try to read from device file */
 	int fileDesc;
 	if ((fileDesc = open(devname, O_RDONLY)) < 0) {
@@ -976,14 +981,9 @@ int main(int argc, char **argv) {
 
 		grab(display, deviceID);		
 
-
-		///* If it is not already running, create thread that listens to XInput events */
-		//if (threadReturn != 0)
-		//	threadReturn = pthread_create(&xLoopThread, NULL,
-		//			xLoopThreadFunction, NULL);
-
 		printf("Reading input from device ... (interrupt to exit)\n");
 
+		/* We perform raw event reading here as X touch events don't seem too reliable */
 		int currentSlot = 0;
 
 		/* If we use the legacy protocol, we collect all data of one finger into tempFingerInfo and set
@@ -1038,6 +1038,7 @@ int main(int argc, char **argv) {
 								/* This messsage indicates we use legacy protocol, so switch */
 								useLegacyProtocol = 1;
 								currentSlot = -1;
+								tempFingerInfo.slotUsed = 0;
 								if(debugMode) printf("Switch to legacy protocol.\n");
 							} else {
 								if(tempFingerInfo.slotUsed) {
